/*
 * csmenu.c channelscan menu
 * writen by reel-multimedia
 *
 * See README for copyright information and
 * how to reach the authors.
 *
 */

#include <iostream>
#include <vector>
#include <string>
#include <linux/dvb/frontend.h> 
#include <sys/ioctl.h> 
#include <vdr/channels.h>
#include <vdr/device.h>
#include <vdr/config.h>
#include <vdr/diseqc.h>
#include <vdr/menu.h>
#include <vdr/menuitems.h>
#include <vdr/tools.h>
#include <vdr/osdbase.h>
#include <vdr/status.h>
#include <vdr/menu.h>
#include "csmenu.h"
#include "filter.h"


#define CHNUMWIDTH 16
#define DVBFRONTEND "/dev/dvb/adapter%d/frontend0"
#define POLLDELAY 2
#define MENUDELAY 3

using std::endl;
using std::time;
using std::string;
using std::cout;
using std::cerr;
using std::vector;
using std::ofstream;

// XXX check all static vars!
typedef vector<int>::const_iterator  iConstIter;

vector<string> tvChannelNames;
vector<string> radioChannelNames;
vector<string> dataChannelNames;

cMutex mutexNames;
cMutex mutexScan;

int cMenuChannelscan::autoSearchStat = 1;
int cMenuChannelscan::sourceStat = 0;

bool cMenuChannelscan::expertSettings = 0;


volatile bool cMenuChannelscan::scanning = false;
bool cMenuChannelscan::scanActiveMenu = false;

// OK
cTransponders *cTransponders::instance_ = NULL;

#define SAT   0
#define CABLE 1
#define TERR  2
 
int pollCount;
#ifndef REELVDR
bool loopMode = false;
#endif

// --- Class cMenuChannelscan ------------------------------------------------

cMenuChannelscan::cMenuChannelscan(int CurrentChannelNr)
:cOsdMenu(tr("Channelscan"), CHNUMWIDTH)
{

  
  data = Setup;
  cTransponders::Create();
  Channels.Save();
  
  frequency = 11837;
  symbolrate = 27500;
  polarization = 'H';
  complete = 0;

  autoSearch = autoSearchStat;
  expertSettings = 0;
#ifndef REELVDR
  loopMode = false;
#endif

  srcTexts[0] = tr("DVB-S - Satellite");
  srcTexts[1] = tr("DVB-C - Cable");
  srcTexts[2] = tr("DVB-T - Terrestrial");

  fecTexts[0] = tr("None");
  fecTexts[1] = "1/2";
  fecTexts[2] = "2/3";
  fecTexts[3] = "3/4";
  fecTexts[4] = "4/5";
  fecTexts[5] = "5/6";
  fecTexts[6] = "6/7";
  fecTexts[7] = "7/8";
  fecTexts[8] = "8/9";
  fecTexts[9] = tr("Auto");

  fecStat = 3;
  //modTexts[0] = "QAM-16";
  //modTexts[1] = "QAM-32";
  modTexts[0] = "QAM-64";
  //modTexts[1] = "QAM-128";
  modTexts[1] = "QAM-256";

  modStat = 0;
   
  //updateChannelsTexts[1] = tr("names only");
  //updateChannelsTexts[3] = tr("add new channels");
  //updateChannelsTexts[4] = tr("add new transponders");
  
  bandwidth = 0; // 0: 7MHz 1: 8MHz

  sRate6875 = 0;
  sRateItem[0] = "6900";
  sRateItem[1] = "6875";
  sRateItem[2] = "6111";

  // CurrentChannel has to be greater than 0!
  CurrentChannelNr = CurrentChannelNr == 0 ? 1 : CurrentChannelNr;
  cChannel *channel = Channels.GetByNumber(CurrentChannelNr);
  if (channel) 
  {
      source = channel->Source();
  }
  else 
  {
     //XXX hotfix  if no channel given test first device for dvb-type!
     source = cSource::FromString("S19.2E");
  }
  
  currentChannel = CurrentChannelNr;
  scanActiveMenu = false;

  pollCount = POLLDELAY;
  loopIndex = 0;
  
  InitLnbs();

  Set();
}

void cMenuChannelscan::InitLnbs()
{
   lnbs = 0;
   for(cDiseqc *diseqc = Diseqcs.First(); diseqc; diseqc=Diseqcs.Next(diseqc))
   {
      if (diseqc != Diseqcs.First() && diseqc->Source() == Diseqcs.Prev(diseqc)->Source() )
          continue;
      loopSources.push_back(diseqc->Source());
      lnbs++;
   }
}



void cMenuChannelscan::Set()
{
    int current = Current();
    Clear();
    
    int blankLines = 4;
   // for DVB-T or DVB-C   
   // we loop through all satalites
   if (loopMode && ::Setup.DiSEqC)  
   {
       Add(new cMenuInfoItem("Scanning configured satellites"));
       AddBlankLineItem(4);

   }
   else
   {
#ifdef REELVDR
      Add(new cMenuEditBoolItem(tr("Setup.DVB$Update channels"),           &data.UpdateChannels, tr("no"), tr("yes") ));
#endif
      Add(new cMenuEditStraItem(tr("Source"), &sourceStat, 3, srcTexts));
      Add(new cMenuEditBoolItem(tr("Search Mode"), &autoSearch, tr("Manual"),  tr("SearchMode$Auto") )); 

      // if Satellite
      if (sourceStat == SAT)
      {
         symbolrate = 27500;  // reset right default values
         Add(new cMenuEditSrcItem(tr("Position"), &source)); //XXX all sources in Diseqc? 
      }
      else if (sourceStat == CABLE && autoSearch)
      {
         Add(new cMenuEditBoolItem(tr("Complete Scan"), &complete, tr("no"), tr("yes"))); //XXX all sources in Diseqc? 
      }
      else  
      {
         AddBlankLineItem(1);
      }

      AddBlankLineItem(1);

      if (autoSearch && sourceStat == CABLE && expertSettings)
      {
           //symbolrate = 6900;  // reset right default values
           Add(new cMenuEditStraItem("Modulation", &modStat,  2, modTexts ));
           //Add(new cMenuEditBoolItem(tr("Symbolrate"), &sRate6875, "6900", "6850"));
           Add(new cMenuEditStraItem(tr("Symbolrate"), &sRate6875, 3, sRateItem));

           blankLines -= 2;
      }
      if (!autoSearch)
      {
          switch (sourceStat)
          { 
              case SAT:
                  Add(new cMenuEditIntItem(tr("Frequency (MHz)"), &frequency));  
                  Add(new cMenuEditChrItem(tr("Polarization"),&polarization, "HVLR"));
                  Add(new cMenuEditIntItem(tr("Symbolrate"), &symbolrate));
                  Add(new cMenuEditStraItem("FEC", &fecStat, 10, fecTexts ));
                  break;
          
              case CABLE:
                  Add(new cMenuEditIntItem(tr("Frequency (kHz)"), &frequency));  
                  Add(new cMenuEditIntItem(tr("Symbolrate"), &symbolrate));
                  Add(new cMenuEditBoolItem(tr("Bandwidth"), &bandwidth, "7 MHz", "8 MHz"));
                  Add(new cMenuEditStraItem("Modulation", &modStat,  2, modTexts ));
                  break;
          
              case TERR:
                  Add(new cMenuEditIntItem(tr("Frequency (kHz)"), &frequency));  
                  Add(new cMenuEditBoolItem(tr("Bandwidth"), &bandwidth, "7 MHz", "8 MHz"));
                  AddBlankLineItem(2);
                  break;
              default: 
                  break;
          }
      }
      else 
         AddBlankLineItem(4);
   }

   // AllignButtom()
   if (::Setup.UseSmallFont || strcmp(::Setup.OSDSkin,"Reel") == 0)
   {
      blankLines -= 1;
   }

   AddBlankLineItem(blankLines);

   Add(new cMenuInfoItem(tr("DiSEqC"), static_cast<bool>(::Setup.DiSEqC))); 

   if (::Setup.DiSEqC)
       DiseqShow();  // 2

   if (!loopMode && sourceStat == CABLE && autoSearch )
   {
       SetHelp(tr("Button$Start") /*tr("DiSEqC"),*/, expertSettings?tr("Standard"):tr("Extended"));
   }
   else 
   {
       SetHelp(tr("Button$Start") /*tr("DiSEqC") */);
   }

   SetCurrent(Get(current));
   Display();
}

void  cMenuChannelscan::Store()
{
     Setup = data;
     Setup.Save();
} 
eOSState cMenuChannelscan::ProcessKey(eKeys Key)
{

  bool HadSub = HasSubMenu();

  int oldAutoSearch = autoSearch;
  int oldSourceStat = sourceStat;
  bool oldExpertSettings = expertSettings;

  eOSState state = cOsdMenu::ProcessKey(Key);

  
// wizard loop - o - matic
  if(loopMode && ::Setup.DiSEqC  && loopIndex <= lnbs)
  {
    if(pollCount <= 0 && !cMenuChannelscan::scanActiveMenu)
    {
        pollCount = POLLDELAY;
        //dsyslog (DBG " PrcKey %d loopIndex %d", pollCount, loopIndex);
        //Load transponder list  and increment index
        cTransponders::GetInstance().Load(loopSources[loopIndex++], false, sRate6875, qam256);
        // Show Scan Menu
        return AddSubMenu(new cMenuScanActive());
    }
    else
    {
      if(!scanActiveMenu)
        pollCount--;
    }
  }
  else if(loopMode && !scanActiveMenu &&  loopIndex >= lnbs)
  {
      //dsyslog (DBG " ~~ loopMode && !scanActiveMenu &&  loopIndex >= lnb ~~ return osCont");
      loopMode = false;
      return osBack;
      /*
      pollCount--;
      if (pollCount == 0)
      { 
         loopMode = false;
         return osBack;
        }
      */
  }
  // wizard loop - o - matic  end

  // items return osUnkown if 
  if (state == osUnknown)
  {
     switch(Key)
     {
        case kOk:
             Store();            
             return osContinue;

        case kRed:
              if (autoSearch)
              {
      		     dsyslog (" kOk Scanning  ");
                 source = sourceStat==1?0x4000:sourceStat==2?0xC000:source;

                 cTransponders::GetInstance().Load(source, complete, sRate6875, modStat); 
                 dsyslog("Source %d complete %d sRate %d qam256 %d", source, complete, sRate6875, modStat);

                 return AddSubMenu(new cMenuScanActive());
      		  }
              else
      		  {
                 source = sourceStat==1?0x4000:sourceStat==2?0xC000:source;

                 cTransponders::GetInstance().Load(source, frequency, symbolrate, bandwidth, modStat, polarization, fecStat);

                 return AddSubMenu(new cMenuScanActive());
      		  }
              break;

       /*
       case  kGreen:
             AddSubMenu(new cMenuSetupLNB);
             //Setup();
             break; */

       case  kGreen: //kYellow:
             if (sourceStat == 1)
             {
                expertSettings  = expertSettings?false:true;
                Set();
             }
             break;

       /*
       case  kYellow: //XXX only for Testing 
             loopMode = loopMode?false:true;
             break;
       */
             
       default: state = osContinue;
     }
  } 

  // forces setup if menu layout should be changed
  if ( Key != kNone && !HadSub)
  {
     
     if (autoSearch != oldAutoSearch ||
         oldExpertSettings != expertSettings || 
         oldSourceStat != sourceStat)
     {
         oldSourceStat = sourceStat;
         Set();
     }
  }
  return state;
}

/*
void cMenuChannelscan::StartScan()
{
  //XXX 
     
}
*/
void cMenuChannelscan::DiseqShow()
{
    for (iConstIter iter = loopSources.begin(); iter != loopSources.end(); ++iter)
    {
      char *buffer = NULL;

      asprintf(&buffer,"LNB %c: %s", 'A'+(iter-loopSources.begin()), *cSource::ToString(*iter));
      //dsyslog ("Show fetch Source [%d] %d", iter-loopSources.begin() , *iter);
      Add(new cMenuInfoItem(buffer));
      free(buffer);
   }
   Add(new cMenuInfoItem(tr("channels in current list"), Channels.MaxNumber()));
} 

cMenuChannelscan::~cMenuChannelscan()
{
  cTransponders::Destroy();
  tvChannelNames.clear();
  radioChannelNames.clear();
  dataChannelNames.clear();
}

// taken fron vdr/menu.c
void cMenuChannelscan::AddBlankLineItem(int lines)
{
   for(int i = 0;i < lines;i++)
   {
      cOsdItem *item = new cOsdItem;
      item->SetSelectable(false);
      item->SetText(strndup(" ", 1), false);
      Add(item);
   }
}

// --- cMenuScanActiveItem ----------------------------------------------------
cMenuScanActiveItem::cMenuScanActiveItem(const char *TvChannel,const char *RadioChannel)
{
  tvChannel = strdup(TvChannel);
  radioChannel = strdup(RadioChannel);
  char *buffer = NULL;
  asprintf(&buffer, "%s \t%s", tvChannel, radioChannel);
  SetText(buffer, false);
  SetSelectable(false);
}

cMenuScanActiveItem::~cMenuScanActiveItem()
{
  free(tvChannel);
  free(radioChannel);
}

// --- cMenuScanActive -------------------------------------------------------

cMenuScanActive::cMenuScanActive()
:cOsdMenu(tr("TV CHANNELS                     RADIO"), 25)
{

#ifdef REELVDR
  oldUpdateChannels = ::Setup.UpdateChannels;
  ::Setup.UpdateChannels = 4;
#endif

  cMenuChannelscan::scanActiveMenu = true;
  oldChannelNumbers = Channels.MaxNumber();

  // Make class 
  tvChannelNames.clear();
  radioChannelNames.clear();

  cTransponders &transponders = cTransponders::GetInstance();
  transponderNum_ = transponders.v_tp_.size();
  Scan.reset(new cScan());

  if (!Scan->StartScanning())
  {
     esyslog (ERR "  Tuner Error");
     // ShowInfo
     cMenuChannelscan::scanning == false;
     //Add(new cMenuInfoItem(tr("Tuner Error")));
  }
  
  Setup();
}

void cMenuScanActive::Setup()
{

  Clear();
  mutexNames.Lock();
  int i = Scan->GetCurrentTp();

  vector<string>::iterator tv;
  vector<string>::iterator radio;
  tv = tvChannelNames.begin();
  radio = radioChannelNames.begin();

  while (tvChannelNames.size() > 10)
  {
    tvChannelNames.erase(tv);
    ++tv;
  }

  while (radioChannelNames.size() > 10)
  {
    radioChannelNames.erase(radio);
    ++radio;
  }

  tv = tvChannelNames.begin();
  radio = radioChannelNames.begin();

  while(1)
  {
     if (tv == tvChannelNames.end() &&  radio == radioChannelNames.end())
        break;

     cMenuScanActiveItem *Item = new cMenuScanActiveItem(tv == tvChannelNames.end()?"":tv->c_str(),
                               radio == radioChannelNames.end()?"":radio->c_str());
     Add(Item);

     if (tv != tvChannelNames.end()) 
		++tv;
     if (radio != radioChannelNames.end()) 
		++radio;

  }
  int nameLines = tvChannelNames.size() > radioChannelNames.size() ? tvChannelNames.size() : radioChannelNames.size();
  mutexNames.Unlock();

  AddBlankLineItem(12 - nameLines);

  /*
    Add(new cMenuScanInfoItem(transponders.Position(),transponders[i].Frequency(),
    transponders[i].Polarisation(), transponders[i].Symbolrate(), transponders[i].FEC()));
  */
 
  if (cMenuChannelscan::scanning)
  {
      Add(new cMenuStatusBar(transponderNum_, i));
      char buffer[50];
      snprintf(buffer,50, tr("Scanning %s\tPlease Wait"), cTransponders::GetInstance().Position().c_str());
      Add(new cMenuInfoItem(buffer));
  }
  else // if (!loopMode) // ScanStat == success 
  {
    Channels.Save();
    if (Channels.MaxNumber() > oldChannelNumbers ) //&& ScanStat == success)
       Add(new cMenuInfoItem(tr("Added new channels"), (Channels.MaxNumber() - oldChannelNumbers)));
     else if (Channels.MaxNumber() == oldChannelNumbers ) //&& ScanStat == success)
       Add(new cMenuInfoItem(tr("No new channels found")));
  }

  Display();
}


eOSState cMenuScanActive::ProcessKey(eKeys Key)
{

  //dsyslog (DBG " !!  MenuActiveScan ProcKey loopMode %s :scanning %s ",
  //loopMode?"Yes":"No", cMenuChannelscan::scanning ?"Yes":"No");
  eOSState state = cOsdMenu::ProcessKey(Key);

  if (!cMenuChannelscan::scanning && loopMode) 
  {
     pollCount--;
     if (pollCount == 0)
     { 
       return osBack;
     }
     Setup();
  }

  if (state == osUnknown)
  {
     switch (Key) 
	 {
        case kOk: return osBack;
        case kBack:
           //dsyslog (" kBack   MenuActiveScan  ");
           cMenuChannelscan::scanning = false; //ScanStat = aborted;
           Skins.Message(mtError, tr("Scanning aborted by User"));
           return osBack;
        default: state = osContinue;
     }
  Setup();
  }
  return state;
}
void cMenuScanActive::AddBlankLineItem(int lines)
{
   for(int i = 0;i < lines;i++)
   {
     cOsdItem *item = new cOsdItem;
     item->SetSelectable(false);
     item->SetText(strndup(" ", 1), false);
     Add(item);
   }
}

cMenuScanActive::~cMenuScanActive()
{
  //XXX 
  tvChannelNames.clear();
  radioChannelNames.clear();
  dataChannelNames.clear();
  // XXX
  cMenuChannelscan::scanning = false;
  cMenuChannelscan::scanActiveMenu = false;
#ifdef REELVDR
  ::Setup.UpdateChannels = oldUpdateChannels;
#endif
}

// cMenuEditSrcItem taken fron vdr/menu.c
// --- cMenuEditSrcItem ------------------------------------------------------

cMenuEditSrcItem::cMenuEditSrcItem(const char *Name, int *Value)
:cMenuEditIntItem(Name, Value, 0)
{
   source = Sources.Get(*Value);
   Set();
}

void cMenuEditSrcItem::Set(void)
{
    if (source) {
        char *buffer = NULL;
        asprintf(&buffer, "%s - %s", *cSource::ToString(source->Code()), source->Description());
        SetValue(buffer);
        free(buffer);
    }
    else
      cMenuEditIntItem::Set();
}

eOSState cMenuEditSrcItem::ProcessKey(eKeys Key)
{
    eOSState state = cMenuEditItem::ProcessKey(Key);

    if (state == osUnknown) {
        if (NORMALKEY(Key) == kLeft) { // TODO might want to increase the delta if repeated quickly?
            if (source && source->Prev()) {
                source = (cSource *)source->Prev();
                *value = source->Code();
            }
        } else if (NORMALKEY(Key) == kRight) {
            if (source) {
                if (source->Next())
                    source = (cSource *)source->Next();
            }
            else
                source = Sources.First();
                if (source)
                    *value = source->Code();
        }
        else
            return state; // we don't call cMenuEditIntItem::ProcessKey(Key) here since we don't accept numerical input

        Set();
        state = osContinue;
    }
    return state;
}

//taken from rotor rotor plugin

cMenuScanInfoItem::cMenuScanInfoItem(string Pos, int f, char pol, int sr, int fecIndex)
{

    const char *pos = Pos.c_str();
    char *buffer = NULL;
    asprintf(&buffer, "%s :\t%s %d %c %d %s",tr("Scanning on transponder"), pos, f, pol, sr, FECToStr(fecIndex));

    SetText(strdup(buffer), false);
    SetSelectable(false);
}
const char *cMenuScanInfoItem::FECToStr(int Index)
{
    switch (Index)
    {
        case 1: return "1/2";
        case 2: return "2/3";
        case 3: return "3/4";
        case 4: return "4/5";
        case 5: return "5/6";
        case 6: return "6/7";
        case 7: return "7/8";
        default : return tr("Auto");
    }
    return tr("Auto");
}

// --- cMenuStatusBar ----------------------------------------------------

cMenuStatusBar::cMenuStatusBar(int Total, int Part)
{
    int barWidth = 70;
    Part+=1;

    if (Part > Total)
    {
      Part = Total;
       cerr << "error! Wrong transponder Values\n";
    }
    if (Total < 1)
    {
      Total = 1;
       cerr << "error! Wrong transponder Values\n";
    }

    proCent = static_cast<int>(((Part) * barWidth/(Total)));

    char buffer[barWidth];
    for(int i = 0;i < proCent; i++)
      buffer[i] = '|';

    for(int i = proCent; i<barWidth; i++)
      buffer[i] = ' ';

   buffer[barWidth] = '\0';
   char *tmp;
    int l = asprintf(&tmp,"%s   %d / %d", buffer, Part, Total);

    SetText(strndup(tmp,l), false);
    SetSelectable(false);
    free(tmp);
}

// --- Class cMenuInfoItem -------------------------------------

cMenuInfoItem::cMenuInfoItem(const char *text, const char *textValue)
{
    char *buffer = NULL;
    asprintf(&buffer,"%s  %s",text, textValue? textValue:"");

    SetText(strdup(buffer), false);
    SetSelectable(false);
   free(buffer);
}

cMenuInfoItem::cMenuInfoItem(const char *text, int intValue)
{
    char *buffer = NULL;
    asprintf(&buffer,"%s: %d",text, intValue);

    SetText(strdup(buffer), false);
    SetSelectable(false);
    free(buffer);
}
cMenuInfoItem::cMenuInfoItem(const char *text, bool boolValue)
{
    char *buffer = NULL;
    asprintf(&buffer,"%s: %s",text, boolValue? tr("enabled"):tr("disabled"));

    SetText(strdup(buffer), false);
    SetSelectable(false);
    free(buffer);
}

